@@ -16,13 +16,29 @@
 #include <bonobo.h>
 
 #include "gnome-cd.h"
+#include "cddb.h"
 #include "cdrom.h"
-#include "callbacks.h"
 #include "display.h"
+#include "callbacks.h"
+#include "preferences.h"
 #include "access/factory.h"
 
 #define DEFAULT_THEME "lcd"
 
+/* Debugging? */
+gboolean debug_mode = FALSE;
+
+void
+gcd_warning (const char *message,
+	     GError *error)
+{
+	if (debug_mode == FALSE) {
+		return;
+	}
+	
+	g_warning (message, error ? error->message : "(None)");
+}
+
 void
 gnome_cd_set_window_title (GnomeCD *gcd,
 			   const char *artist,
@@ -30,9 +46,9 @@ gnome_cd_set_window_title (GnomeCD *gcd,
 {
 	char *title;
 
-	if (artist == NULL &&
+	if (artist == NULL ||
 	    track == NULL) {
-		title = g_strdup ("Gnome-CD " VERSION);
+		title = g_strdup (_("Gnome CD Player"));
 	} else {
 		title = g_strconcat (artist, " - ", track, NULL);
 	}
@@ -76,7 +92,7 @@ skip_to_track (GtkWidget *item,
 	}
 	
 	if (gnome_cdrom_play (GNOME_CDROM (gcd->cdrom), track + 1, &msf, end_track, endmsf, &error) == FALSE) {
-		g_warning ("Error skipping %s", error->message);
+		gcd_warning ("Error skipping %s", error);
 		g_error_free (error);
 	}
 }
@@ -96,7 +112,8 @@ gnome_cd_build_track_list_menu (GnomeCD *gcd)
 	}
 
 	menu = GTK_MENU(gtk_menu_new ());
-	if (gcd->disc_info != NULL) {
+	if (gcd->disc_info != NULL &&
+	    gcd->disc_info->track_info) {
 		int i;
 		for (i = 0; i < gcd->disc_info->ntracks; i++) {
 			char *title;
@@ -181,26 +198,6 @@ make_button_from_widget (GnomeCD *gcd,
 	return button;
 }
 
-static GtkWidget *
-make_button_from_file (GnomeCD *gcd,
-		       const char *filename,
-		       GCallback func,
-		       const char *tooltip,
-		       const char *shortname)
-{
-	GtkWidget *pixmap;
-	char *fullname;
-
-	fullname = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
-		   filename, TRUE, NULL);
-	g_return_val_if_fail (fullname != NULL, NULL);
-
-	pixmap = gtk_image_new_from_file (fullname);
-	g_free (fullname);
-
-	return make_button_from_widget (gcd, pixmap, func, tooltip, shortname);
-}
-
 static GtkWidget *
 make_button_from_pixbuf (GnomeCD *gcd,
 			 GdkPixbuf *pixbuf,
@@ -279,7 +276,7 @@ window_destroy_cb (GtkWidget *window,
 	}
 
 	/* Unref the cddb slave */
-	close_cddb_client ();
+	cddb_close_client ();
 
 	/* And the track editor */
 	destroy_track_editor ();
@@ -314,7 +311,7 @@ make_popup_menu (GnomeCD *gcd)
 	for (i = 0; menuitems[i].name != NULL; i++) {
 		GtkWidget *item, *image;
 
-		item = gtk_image_menu_item_new_with_mnemonic (menuitems[i].name);
+		item = gtk_image_menu_item_new_with_mnemonic (_(menuitems[i].name));
 		if (menuitems[i].icon != NULL) {
 			char *ext = strrchr (menuitems[i].icon, '.');
 
@@ -360,13 +357,10 @@ init_player (void)
 	GnomeCD *gcd;
 	GnomeCDRomStatus *status;
 	GtkWidget *display_box;
-	GtkWidget *top_hbox, *button_hbox, *side_vbox;
-	GtkWidget *button, *arrow;
+	GtkWidget *top_hbox, *button_hbox, *option_hbox;
+	GtkWidget *button;
 	GdkPixbuf *pixbuf;
 	GError *error = NULL;
-	GtkAdjustment *adj;
-	GtkWidget *slider;
-	char *fullname;
 
 	gcd = g_new0 (GnomeCD, 1);
 
@@ -404,7 +398,7 @@ init_player (void)
 	gcd->cdrom = gnome_cdrom_new (gcd->preferences->device, GNOME_CDROM_UPDATE_CONTINOUS, &error);
 	if (gcd->cdrom == NULL) {
 		if (error != NULL) {
-			g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
+			gcd_warning ("%s", error);
 			g_error_free (error);
 		}
 	} else {
@@ -417,9 +411,9 @@ init_player (void)
 		
 		dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_NONE,
-						 _("The CD device that is set is not a CDRom drive. This means that GnomeCD\n"
-						   "will be unable to run correctly. Press Set device to go to a dialog\n"
-						   "where you can set the device, or click Quit to quit GnomeCD"));
+						 _("%s\nThis means that GnomeCD"
+						   " will be unable to run correctly. Press Set device to go to a dialog "
+						   "where you can set the device, or click Quit to quit GnomeCD"), error->message);
 		gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_QUIT, GTK_RESPONSE_CLOSE,
 					_("Set device"), 1, NULL);
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), 1);
@@ -444,7 +438,7 @@ init_player (void)
 	}
 		
 	gcd->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_title (GTK_WINDOW (gcd->window), "Gnome-CD "VERSION);
+	gtk_window_set_title (GTK_WINDOW (gcd->window), _("Gnome CD Player"));
 	gtk_window_set_wmclass (GTK_WINDOW (gcd->window), "main_window", "gnome-cd");
 	gtk_window_set_default_size (GTK_WINDOW (gcd->window), 350, 129);
 	
@@ -458,22 +452,9 @@ init_player (void)
 
 	g_signal_connect (G_OBJECT (gcd->window), "destroy",
 			  G_CALLBACK (window_destroy_cb), gcd);
-	gcd->vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
+	gcd->vbox = gtk_vbox_new (FALSE, 1);
 	gcd->tooltips = gtk_tooltips_new ();
-	top_hbox = gtk_hbox_new (FALSE, 0);
-
-	/* Create app controls */
-	side_vbox = gtk_vbox_new (FALSE, 0);
-	button = make_button_from_stock (gcd, GTK_STOCK_INDEX, G_CALLBACK(open_track_editor), _("Open track editor"), _("Track editor"));
-	gtk_widget_set_sensitive (button, FALSE);
-	gtk_box_pack_start (GTK_BOX (side_vbox), button, FALSE, FALSE, 0);
-	gcd->trackeditor_b = button;
-
-	button = make_button_from_stock (gcd, GTK_STOCK_PREFERENCES, G_CALLBACK(open_preferences), _("Preferences"), _("Preferences"));
-	gtk_box_pack_start (GTK_BOX (side_vbox), button, FALSE, FALSE, 0);
-	gcd->properties_b = button;
-
-	gtk_box_pack_start (GTK_BOX (top_hbox), side_vbox, FALSE, FALSE, 0);
+	top_hbox = gtk_hbox_new (FALSE, 1);
 
 	/* Create the display */
 	display_box = gtk_vbox_new (FALSE, 1);
@@ -495,11 +476,6 @@ init_player (void)
 	
 	gtk_box_pack_start (GTK_BOX (display_box), gcd->display, TRUE, TRUE, 0);
 
-	gcd->tracks = gtk_option_menu_new ();
-	g_signal_connect (G_OBJECT (gcd->tracks), "changed",
-			  G_CALLBACK (skip_to_track), gcd);
-	gnome_cd_build_track_list_menu (gcd);
-	gtk_box_pack_start (GTK_BOX (display_box), gcd->tracks, FALSE, FALSE, 0);
 
 	gtk_box_pack_start (GTK_BOX (top_hbox), display_box, TRUE, TRUE, 0);
 
@@ -508,19 +484,51 @@ init_player (void)
 	gtk_range_set_inverted (GTK_RANGE (gcd->slider), TRUE);
 	gtk_scale_set_draw_value (GTK_SCALE (gcd->slider), FALSE);
 	gtk_box_pack_start (GTK_BOX (top_hbox), gcd->slider, FALSE, FALSE, 0);
+	
+	gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET (gcd->slider),
+			      _("Volume Control"), NULL);
 
+	gtk_box_pack_start (GTK_BOX (gcd->vbox), top_hbox, TRUE, TRUE, 0);
+
+	option_hbox = gtk_hbox_new (FALSE, 2);
+
+	/* Create app controls */
+	button = make_button_from_stock (gcd, GTK_STOCK_INDEX,
+					 G_CALLBACK(open_track_editor),
+					 _("Open track editor"),
+					 _("Track editor"));
+	gtk_widget_set_sensitive (button, FALSE);
+	gtk_box_pack_start (GTK_BOX (option_hbox), button, FALSE, FALSE, 0);
+	gcd->trackeditor_b = button;
+
+	button = make_button_from_stock (gcd, GTK_STOCK_PREFERENCES,
+					 G_CALLBACK(open_preferences),
+					 _("Open preferences"),
+					 _("Preferences"));
+	gtk_box_pack_start (GTK_BOX (option_hbox), button, FALSE, FALSE, 0);
+	gcd->properties_b = button;
+
+	gcd->tracks = gtk_option_menu_new ();
+	g_signal_connect (G_OBJECT (gcd->tracks), "changed",
+			  G_CALLBACK (skip_to_track), gcd);
+	gtk_tooltips_set_tip (gcd->tooltips, GTK_WIDGET (gcd->tracks), 
+			      _("Track List"), NULL );
+	gnome_cd_build_track_list_menu (gcd);
+	gtk_box_pack_start (GTK_BOX (option_hbox), gcd->tracks, TRUE, TRUE, 0);
+
+	gtk_box_pack_start (GTK_BOX (gcd->vbox), option_hbox, FALSE, FALSE, 0);
+	
 	/* Get the initial volume */
 	if (gnome_cdrom_get_status (gcd->cdrom, &status, NULL) == TRUE) {
 		gtk_range_set_value (GTK_RANGE (gcd->slider),
 				     (double) status->volume);
 		g_free (status);
 	} else {
-		g_warning ("Error getting status");
+		gcd_warning ("Error getting status: %s", NULL);
 	}
 
 	g_signal_connect (G_OBJECT (gcd->slider), "value-changed",
 			  G_CALLBACK (volume_changed), gcd);
-	gtk_box_pack_start (GTK_BOX (gcd->vbox), top_hbox, TRUE, TRUE, 0);
 	
 	button_hbox = gtk_hbox_new (TRUE, 2);
 	
@@ -607,19 +615,23 @@ static gint client_die(GnomeClient *client,
 
 
 int 
-main (int argc,
-      char *argv[])
+main (int argc, char *argv[])
 {
 	GnomeCD *gcd;
 	GnomeClient *client;
 
+	free (malloc (8)); /* -lefence */
+
+	if (g_getenv ("GNOME_CD_DEBUG")) {
+		debug_mode = TRUE;
+	}
+	
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
 	gnome_program_init ("Gnome-CD", VERSION, LIBGNOMEUI_MODULE, 
 			    argc, argv, NULL);
-	gconf_init (argc, argv, NULL);
 	client = gnome_master_client ();
     	g_signal_connect (client, "save_yourself",
                          G_CALLBACK (save_session), (gpointer) argv[0]);
@@ -659,7 +671,7 @@ main (int argc,
 	
 	gtk_widget_show (gcd->window);
 
-	setup_factory();
+	setup_a11y_factory ();
 
 	bonobo_main ();
 	return 0;
