@@ -14,6 +14,7 @@
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <sys/cdio.h>
+#include <sys/audioio.h>	
 #include <errno.h>
 
 #include "gnome-cd.h"
@@ -30,11 +31,6 @@ typedef struct _SolarisCDRomTrackInfo {
 } SolarisCDRomTrackInfo;
 
 struct _SolarisCDRomPrivate {
-	char *cdrom_device;
-	guint32 update_id;
-	int cdrom_fd;
-	int ref_count;
-
 	GnomeCDRomUpdate update;
 
 	struct cdrom_tochdr *tochdr;
@@ -42,8 +38,6 @@ struct _SolarisCDRomPrivate {
 	unsigned char track0, track1;
 
 	SolarisCDRomTrackInfo *track_info;
-	
-	GnomeCDRomStatus *recent_status;
 };
 
 static gboolean solaris_cdrom_eject (GnomeCDRom *cdrom,
@@ -75,20 +69,11 @@ static gboolean solaris_cdrom_close_tray (GnomeCDRom *cdrom,
 static GnomeCDRomMSF blank_msf = { 0, 0, 0};
 
 static void
-finalize (GObject *object)
+solaris_cdrom_finalize (GObject *object)
 {
-	SolarisCDRom *cdrom;
-
-	cdrom = SOLARIS_CDROM (object);
-	if (cdrom->priv == NULL) {
-		return;
-	}
-
-	close (cdrom->priv->cdrom_fd);
-	g_free (cdrom->priv->cdrom_device);
+	SolarisCDRom *cdrom = (SolarisCDRom *) object;
 
 	g_free (cdrom->priv);
-	cdrom->priv = NULL;
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -129,47 +114,15 @@ add_msf (GnomeCDRomMSF *msf1,
 
 static gboolean
 solaris_cdrom_open (SolarisCDRom *lcd,
-		  GError **error)
+		    GError      **error)
 {
-	if (lcd->priv->cdrom_fd != -1) {
-		lcd->priv->ref_count++;
-		return TRUE;
-	}
-
-	lcd->priv->cdrom_fd = open (lcd->priv->cdrom_device, O_RDONLY | O_NONBLOCK);
-	if (lcd->priv->cdrom_fd < 0) {
-		if (error) {
-			*error = g_error_new (GNOME_CDROM_ERROR,
-					      GNOME_CDROM_ERROR_NOT_OPENED,
-					      "Unable to open %s. This may be caused by:\n"
-					      "a) CD support is not compiled into Solaris\n"
-					      "b) You do not have the correct permissions to access the CD drive\n"
-					      "c) There is no device in the cd drive.\n",					     
-					      lcd->priv->cdrom_device);
-		}
-
-		return FALSE;
-	}
-
-	lcd->priv->ref_count = 1;
-	return TRUE;
+	return gnome_cdrom_open_dev (GNOME_CDROM (lcd), error);
 }
 
 static void
 solaris_cdrom_close (SolarisCDRom *lcd)
 {
-	if (lcd->priv->cdrom_fd == -1) {
-		return;
-	}
-
-	lcd->priv->ref_count--;
-
-	if (lcd->priv->ref_count <= 0) {
-		close (lcd->priv->cdrom_fd);
-		lcd->priv->cdrom_fd = -1;
-	}
-
-	return;
+	gnome_cdrom_close_dev (GNOME_CDROM (lcd), FALSE);
 }
 
 static void
@@ -205,8 +158,9 @@ calculate_track_lengths (SolarisCDRom *lcd)
 }
 
 static void
-solaris_cdrom_update_cd (SolarisCDRom *lcd)
+solaris_cdrom_update_cd (GnomeCDRom *cdrom)
 {
+	SolarisCDRom *lcd = SOLARIS_CDROM (cdrom);
 	SolarisCDRomPrivate *priv;
 	struct cdrom_tocentry tocentry;
 	int i, j;
@@ -219,7 +173,7 @@ solaris_cdrom_update_cd (SolarisCDRom *lcd)
 		return;
 	}
 
-	if (ioctl (priv->cdrom_fd, CDROMREADTOCHDR, priv->tochdr) < 0) {
+	if (ioctl (cdrom->fd, CDROMREADTOCHDR, priv->tochdr) < 0) {
 		g_warning ("Error reading CD header");
 		solaris_cdrom_close (lcd);
 
@@ -236,7 +190,7 @@ solaris_cdrom_update_cd (SolarisCDRom *lcd)
 		tocentry.cdte_track = j;
 		tocentry.cdte_format = CDROM_MSF;
 
-		if (ioctl (priv->cdrom_fd, CDROMREADTOCENTRY, &tocentry) < 0) {
+		if (ioctl (cdrom->fd, CDROMREADTOCENTRY, &tocentry) < 0) {
 			g_warning ("IOCtl failed");
 			continue;
 		}
@@ -248,7 +202,7 @@ solaris_cdrom_update_cd (SolarisCDRom *lcd)
 
 	tocentry.cdte_track = CDROM_LEADOUT;
 	tocentry.cdte_format = CDROM_MSF;
-	if (ioctl (priv->cdrom_fd, CDROMREADTOCENTRY, &tocentry) < 0) {
+	if (ioctl (cdrom->fd, CDROMREADTOCENTRY, &tocentry) < 0) {
 		g_warning ("Error getting leadout");
 		solaris_cdrom_invalidate (lcd);
 		return;
@@ -285,7 +239,7 @@ solaris_cdrom_eject (GnomeCDRom *cdrom,
 	}
 
 	if (status->cd != GNOME_CDROM_STATUS_TRAY_OPEN) {
-		if (ioctl (lcd->priv->cdrom_fd, CDROMEJECT, 0) < 0) {
+		if (ioctl (cdrom->fd, CDROMEJECT, 0) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
@@ -309,8 +263,9 @@ solaris_cdrom_eject (GnomeCDRom *cdrom,
 	}
 
 	g_free (status);
-	lcd->priv->ref_count = 0;
-	solaris_cdrom_close (lcd);
+
+	gnome_cdrom_close_dev (cdrom, TRUE);
+
 	return TRUE;
 }
 
@@ -541,7 +496,7 @@ solaris_cdrom_play (GnomeCDRom *cdrom,
 		}
 
 		/* PLAY IT AGAIN */
-		if (ioctl (priv->cdrom_fd, CDROMPLAYMSF, &msf) < 0) {
+		if (ioctl (cdrom->fd, CDROMPLAYMSF, &msf) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
@@ -590,7 +545,7 @@ solaris_cdrom_pause (GnomeCDRom *cdrom,
 	}
 
 	if (status->audio == GNOME_CDROM_AUDIO_PAUSE) {
-		if (ioctl (lcd->priv->cdrom_fd, CDROMRESUME) < 0) {
+		if (ioctl (cdrom->fd, CDROMRESUME) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
@@ -609,7 +564,7 @@ solaris_cdrom_pause (GnomeCDRom *cdrom,
 	}
 
 	if (status->audio == GNOME_CDROM_AUDIO_PLAY) {
-		if (ioctl (lcd->priv->cdrom_fd, CDROMPAUSE, 0) < 0) {
+		if (ioctl (cdrom->fd, CDROMPAUSE, 0) < 0) {
 			if (error) {
 				*error = g_error_new (GNOME_CDROM_ERROR,
 						      GNOME_CDROM_ERROR_SYSTEM_ERROR,
@@ -655,7 +610,7 @@ solaris_cdrom_stop (GnomeCDRom *cdrom,
 	}
 #endif
 
-	if (ioctl (lcd->priv->cdrom_fd, CDROMSTOP, 0) < 0) {
+	if (ioctl (cdrom->fd, CDROMSTOP, 0) < 0) {
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
@@ -800,6 +755,8 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 	SolarisCDRomPrivate *priv;
 	GnomeCDRomStatus *realstatus;
 	struct cdrom_subchnl subchnl;
+	struct audio_info audioinfo;
+	int vol_fd;
 	int cd_status;
 
 	g_return_val_if_fail (status != NULL, TRUE);
@@ -807,8 +764,9 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 	lcd = SOLARIS_CDROM (cdrom);
 	priv = lcd->priv;
 
-	*status = g_new (GnomeCDRomStatus, 1);
+	*status = g_new0 (GnomeCDRomStatus, 1);
 	realstatus = *status;
+	realstatus->volume = 0;
 
 	if (solaris_cdrom_open (lcd, error) == FALSE) {
 		g_free (realstatus);
@@ -817,7 +775,7 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 	}
 
 #if 0
-	cd_status = ioctl (priv->cdrom_fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
+	cd_status = ioctl (cdrom->fd, CDROM_DRIVE_STATUS, CDSL_CURRENT);
 	if (cd_status != -1) {
 		switch (cd_status) {
 		case CDS_NO_INFO:
@@ -873,7 +831,7 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 #endif
 
 	subchnl.cdsc_format = CDROM_MSF;
-	if (ioctl (priv->cdrom_fd, CDROMSUBCHNL, &subchnl) < 0) {
+	if (ioctl (cdrom->fd, CDROMSUBCHNL, &subchnl) < 0) {
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
@@ -885,6 +843,17 @@ solaris_cdrom_get_status (GnomeCDRom *cdrom,
 		g_free (realstatus);
 		return FALSE;
 	}
+	/* get initial volume */
+	vol_fd = open ( "/dev/audioctl", O_RDWR);
+	if (ioctl (vol_fd, AUDIO_GETINFO, &audioinfo) < 0) {
+			g_warning ("(solaris_cdrom_get_status): AUDIO_GETINFO ioctl failed %s",
+				   strerror (errno));
+			realstatus->volume = -1;
+	} else {
+		realstatus->volume = audioinfo.play.gain;
+	}
+
+	close (vol_fd);
 
 	solaris_cdrom_close (lcd);
 
@@ -941,7 +910,7 @@ solaris_cdrom_close_tray (GnomeCDRom *cdrom,
 	}
 
 #if 0
-	if (ioctl (lcd->priv->cdrom_fd, CDROMCLOSETRAY) < 0) {
+	if (ioctl (cdrom->fd, CDROMCLOSETRAY) < 0) {
 		if (error) {
 			*error = g_error_new (GNOME_CDROM_ERROR,
 					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
@@ -958,12 +927,91 @@ solaris_cdrom_close_tray (GnomeCDRom *cdrom,
 	return TRUE;
 }
 
+static gboolean
+solaris_cdrom_set_volume (GnomeCDRom *cdrom,
+			  int volume,
+                          GError **error)
+{
+	SolarisCDRom *lcd;
+	SolarisCDRomPrivate *priv;
+	struct audio_info audioinfo;
+	struct cdrom_volctrl vol;
+	int vol_fd;
+
+	lcd = SOLARIS_CDROM (cdrom);
+	priv = lcd->priv;
+	AUDIO_INITINFO (&audioinfo)
+
+	audioinfo.play.gain = volume;
+
+	vol_fd = open ("/dev/audioctl", O_RDWR);
+	if (vol_fd < 0) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+                                              GNOME_CDROM_ERROR_SYSTEM_ERROR,
+                                              "(solaris_cdrom_set_volume:1): ioctl failed %s",
+                                              strerror (errno));
+		}
+		close (vol_fd);
+		return FALSE;
+	}
+	
+	if (ioctl (vol_fd, AUDIO_SETINFO, &audioinfo) < 0) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+                                              GNOME_CDROM_ERROR_SYSTEM_ERROR,
+                                              "(solaris_cdrom_set_volume:1): ioctl failed %s",
+					      strerror (errno));
+		}
+
+		close (vol_fd);
+		return FALSE;
+	}
+
+	close (vol_fd);
+
+	if (solaris_cdrom_open (lcd, error) == FALSE) {
+		return FALSE;
+	}
+
+	vol.channel0 = volume;
+	vol.channel1 = vol.channel2 = vol.channel3 = volume;
+
+	if (ioctl (cdrom->fd, CDROMVOLCTRL, &vol) < 0) {
+		if (error) {
+			*error = g_error_new (GNOME_CDROM_ERROR,
+					      GNOME_CDROM_ERROR_SYSTEM_ERROR,
+              				      "(solaris_cdrom_set_volume:1): ioctl failed %s",
+					      strerror (errno));
+                }
+
+		solaris_cdrom_close (lcd);
+		return FALSE;
+	}
+
+	solaris_cdrom_close (lcd);
+	return TRUE;
+}
+
 static gboolean
 solaris_cdrom_is_cdrom_device (GnomeCDRom *cdrom,
 			       const char *device,
 			       GError **error)
 {
 	int fd;
+	int vol_fd;
+	int volume;
+	struct audio_info audioinfo;
+	struct cdrom_volctrl vol;
+
+	vol_fd = open ( "/dev/audioctl", O_RDWR);
+	if (ioctl (vol_fd, AUDIO_GETINFO, &audioinfo) < 0) {
+			g_warning ("(solaris_cdrom_is_cdrom_device): AUDIO_GETINFO ioctl failed %s",
+                                   strerror (errno));
+	} else {
+                volume = audioinfo.play.gain;
+	}
+	close (vol_fd);
 
 	if (device == NULL || *device == 0) {
 		return FALSE;
@@ -974,8 +1022,10 @@ solaris_cdrom_is_cdrom_device (GnomeCDRom *cdrom,
 		return FALSE;
 	}
 
+	vol.channel0 = volume;
+	vol.channel1 = vol.channel2 = vol.channel3 = volume;
 	/* Fire a harmless ioctl at the device. */
-	if (ioctl (fd, CDROMSTOP, 0) < 0) {
+	if (ioctl (fd, CDROMVOLCTRL, &vol) < 0) {
 		/* Failed, it's not a CDROM drive */
 		g_print ("%s is not a CDROM drive", device);
 		close (fd);
@@ -1010,7 +1060,7 @@ solaris_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 		return TRUE;
 	}
 	
-	*data = g_new (GnomeCDRomCDDBData, 1);
+	*data = g_new0 (GnomeCDRomCDDBData, 1);
 
 	for (i = 0; i < priv->number_tracks; i++) {
 		n += cddb_sum ((priv->track_info[i].address.minute * 60) + 
@@ -1024,7 +1074,7 @@ solaris_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 	(*data)->discid = ((n % 0xff) << 24 | t << 8 | (priv->track1));
 	(*data)->ntrks = priv->track1;
 	(*data)->nsecs = (priv->track_info[priv->track1].address.minute * 60) + priv->track_info[priv->track1].address.second;
-	(*data)->offsets = g_new (unsigned int, priv->track1 + 1);
+	(*data)->offsets = g_new0 (unsigned int, priv->track1 + 1);
 
 	for (i = priv->track0 - 1; i < priv->track1; i++) {
 		(*data)->offsets[i] = msf_to_frames (&priv->track_info[i].address);
@@ -1035,58 +1085,8 @@ solaris_cdrom_get_cddb_data (GnomeCDRom *cdrom,
 	return TRUE;
 }
 
-static gboolean
-solaris_cdrom_set_device (GnomeCDRom *cdrom,
-			const char *device,
-			GError **error)
-{
-	SolarisCDRom *lcd;
-	SolarisCDRomPrivate *priv;
-	GnomeCDRomStatus *status;
-	
-	lcd = SOLARIS_CDROM (cdrom);
-	priv = lcd->priv;
-
-	if (strcmp (priv->cdrom_device, device) == 0) {
-		/* device is the same */
-		return TRUE;
-	}
-	
-	if (solaris_cdrom_get_status (cdrom, &status, NULL) == TRUE) {
-		if (status->audio == GNOME_CDROM_AUDIO_PLAY) {
-			if (solaris_cdrom_stop (cdrom, &error) == FALSE) {
-				return FALSE;
-			}
-		}
-	}
-
-	if (priv->cdrom_device != NULL) {
-		g_free (priv->cdrom_device);
-	}
-	priv->cdrom_device = g_strdup (device);
-	
-	if (gnome_cdrom_is_cdrom_device (GNOME_CDROM (lcd),
-					 priv->cdrom_device, error) == FALSE) {
-		if (error) {
-			*error = g_error_new (GNOME_CDROM_ERROR,
-					      GNOME_CDROM_ERROR_NOT_OPENED,
-					      "%s does not point to a valid CDRom device. This may be caused by:\n"
-					      "a) CD support is not compiled into Linux\n"
-					      "b) You do not have the correct permissions to access the CD drive\n"
-					      "c) %s is not the CD drive.\n",					     
-					      priv->cdrom_device, priv->cdrom_device);
-		}
-	}
-
-	/* Force the new CD to be scanned at next update cycle */
-	g_free (priv->recent_status);
-	priv->recent_status = NULL;
-
-	return TRUE;
-}
-
 static void
-class_init (SolarisCDRomClass *klass)
+solaris_cdrom_class_init (SolarisCDRomClass *klass)
 {
 	GObjectClass *object_class;
 	GnomeCDRomClass *cdrom_class;
@@ -1094,7 +1094,7 @@ class_init (SolarisCDRomClass *klass)
 	object_class = G_OBJECT_CLASS (klass);
 	cdrom_class = GNOME_CDROM_CLASS (klass);
 
-	object_class->finalize = finalize;
+	object_class->finalize = solaris_cdrom_finalize;
 
 	cdrom_class->eject = solaris_cdrom_eject;
 	cdrom_class->next = solaris_cdrom_next;
@@ -1106,80 +1106,24 @@ class_init (SolarisCDRomClass *klass)
 	cdrom_class->back = solaris_cdrom_back;
 	cdrom_class->get_status = solaris_cdrom_get_status;
 	cdrom_class->close_tray = solaris_cdrom_close_tray;
+	cdrom_class->set_volume = solaris_cdrom_set_volume;
 	cdrom_class->is_cdrom_device = solaris_cdrom_is_cdrom_device;
+	cdrom_class->update_cd = solaris_cdrom_update_cd;
 	
 	/* For CDDB */
   	cdrom_class->get_cddb_data = solaris_cdrom_get_cddb_data;
 
-	cdrom_class->set_device = solaris_cdrom_set_device;
-	
 	parent_class = g_type_class_peek_parent (klass);
 }
 
 static void
-init (SolarisCDRom *cdrom)
+solaris_cdrom_init (SolarisCDRom *cdrom)
 {
-	cdrom->priv = g_new (SolarisCDRomPrivate, 1);
-	cdrom->priv->tochdr = g_new (struct cdrom_tochdr, 1);
-	cdrom->priv->cdrom_fd = -1;
-	cdrom->priv->ref_count = 0;
-	cdrom->priv->update_id = -1;
-	cdrom->priv->recent_status = NULL;
+	cdrom->priv = g_new0 (SolarisCDRomPrivate, 1);
+	cdrom->priv->tochdr = g_new0 (struct cdrom_tochdr, 1);
 	cdrom->priv->track_info = NULL;
 }
 
-static gboolean
-update_cd (gpointer data)
-{
-	SolarisCDRom *lcd;
-	SolarisCDRomPrivate *priv;
-	GnomeCDRomStatus *status;
-	GError *error;
-
-	lcd = data;
-	priv = lcd->priv;
-
-	/* Do an update */
-	if (solaris_cdrom_get_status (GNOME_CDROM (lcd), &status, &error) == FALSE) {
-		g_warning ("%s: %s", G_GNUC_FUNCTION, error->message);
-
-		g_error_free (error);
-		return TRUE;
-	}
-
-	if (priv->recent_status == NULL) {
-		priv->recent_status = status;
-		
-		/* emit changed signal */
-		solaris_cdrom_update_cd (lcd);
-		if (priv->update != GNOME_CDROM_UPDATE_NEVER) {
-			gnome_cdrom_status_changed (GNOME_CDROM (lcd), priv->recent_status);
-		}
-	} else {
-		if (gnome_cdrom_status_equal (status, priv->recent_status) == TRUE) {
-			if (priv->update == GNOME_CDROM_UPDATE_CONTINOUS) {
-				/* emit changed signal */
-				gnome_cdrom_status_changed (GNOME_CDROM (lcd), priv->recent_status);
-			}
-		} else {
-			if (priv->recent_status->cd != GNOME_CDROM_STATUS_OK &&
-			    status->cd == GNOME_CDROM_STATUS_OK) {
-				solaris_cdrom_update_cd (lcd);
-			}
-
-			g_free (priv->recent_status);
-			priv->recent_status = status;
-
-			if (priv->update != GNOME_CDROM_UPDATE_NEVER) {
-				/* emit changed signal */
-				gnome_cdrom_status_changed (GNOME_CDROM (lcd), priv->recent_status);
-			}
-		}
-	}
-
-	return TRUE;
-}
-	
 /* API */
 GType
 solaris_cdrom_get_type (void)
@@ -1189,8 +1133,8 @@ solaris_cdrom_get_type (void)
 	if (type == 0) {
 		GTypeInfo info = {
 			sizeof (SolarisCDRomClass),
-			NULL, NULL, (GClassInitFunc) class_init, NULL, NULL,
-			sizeof (SolarisCDRom), 0, (GInstanceInitFunc) init,
+			NULL, NULL, (GClassInitFunc) solaris_cdrom_class_init, NULL, NULL,
+			sizeof (SolarisCDRom), 0, (GInstanceInitFunc) solaris_cdrom_init,
 		};
 
 		type = g_type_register_static (GNOME_CDROM_TYPE, "SolarisCDRom", &info, 0);
@@ -1199,56 +1143,12 @@ solaris_cdrom_get_type (void)
 	return type;
 }
 
-/* This is the creation function. It returns a GnomeCDRom object instead of
-   a SolarisCDRom one because the caller doesn't know about SolarisCDRom only
-   GnomeCDRom */
 GnomeCDRom *
-gnome_cdrom_new (const char *cdrom_device,
+gnome_cdrom_new (const char      *cdrom_device,
 		 GnomeCDRomUpdate update,
-		 GError **error)
+		 GError         **error)
 {
-	SolarisCDRom *cdrom;
-	SolarisCDRomPrivate *priv;
-	int fd;
-
-	cdrom = g_object_new (solaris_cdrom_get_type (), NULL);
-	priv = cdrom->priv;
-
-	priv->cdrom_device = g_strdup (cdrom_device);
-	priv->update = update;
-
-	solaris_cdrom_open (cdrom, error);
-	solaris_cdrom_close (cdrom);
-	if (gnome_cdrom_is_cdrom_device (GNOME_CDROM (cdrom),
-					 cdrom->priv->cdrom_device,
-					 error) == FALSE) {
-		if (error) {
-			*error = g_error_new (GNOME_CDROM_ERROR,
-					      GNOME_CDROM_ERROR_NOT_OPENED,
-					      "%s does not point to a valid CDRom device. This may be caused by:\n"
-					      "a) CD support is not compiled into Linux\n"
-					      "b) You do not have the correct permissions to access the CD drive\n"
-					      "c) %s is not the CD drive.\n",					     
-					      cdrom->priv->cdrom_device, cdrom->priv->cdrom_device);
-		}
-	}
-	
-	/* Force an update so that a status will always exist */
-	update_cd (cdrom);
-	
-	/* All worked, start the counter */
-	switch (update) {
-	case GNOME_CDROM_UPDATE_NEVER:
-		break;
-		
-	case GNOME_CDROM_UPDATE_WHEN_CHANGED:
-	case GNOME_CDROM_UPDATE_CONTINOUS:
-		priv->update_id = g_timeout_add (1000, update_cd, cdrom);
-		break;
-		
-	default:
-		break;
-	}
-
-	return GNOME_CDROM (cdrom);
+	return gnome_cdrom_construct (
+		g_object_new (solaris_cdrom_get_type (), NULL),
+		cdrom_device, update, error);
 }
